import 'package:flutter/material.dart';
import 'package:osam/osam.dart';

part 'main.g.dart';

@HiveType(typeId: 0)
class Counter {
  @HiveField(0)
  final PropertyNotifier<int> _value;
  IProperty<int> get value => _value;

  Counter({PropertyNotifier<int> value}) : _value = value ?? PropertyNotifier(0);

  void increment() => _value.value++;
}

final persist = Persist<Counter, Null>(appName: '123');

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  Hive.registerAdapter(CounterAdapter());
  registerOsamAdapters();
  await persist.init();
  final counter = persist.appState ?? Counter();

  runApp(Example(
    counter: counter,
  ));
}

class Example extends StatelessWidget {
  final Counter counter;

  const Example({Key key, this.counter})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        floatingActionButton: FloatingActionButton(
          child: Icon(Icons.add),
          onPressed: () {
            counter.increment();
            persist.save(state: counter, uiState: null, version: null);
          },
        ),
        body: Center(
          child: ValueListenableBuilder(
            valueListenable: counter.value,
            builder: (_, value, __) => Text(
              value.toString(),
              style: const TextStyle(fontSize: 40),
            ),
          ),
        ),
      ),
    );
  }
}
